// ----------------------------------------------------------------------------------
//
// File Name	: 'lcd_lib.c'
// Title		: 4 bit LCd interface
// Author		: Scienceprog.com - Copyright (C) 2007 : Revised: 2007-08-08
// Modified		: 2008-07-02	
//
// ----------------------------------------------------------------------------------
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "lcd.h"
static char str_a[8]="        ", temp[8];	// as those are used all the time
// ----------------------------------------------------------------------------------
void LCDsendChar(char ch){		//Sends Char to LCD

	//4 bit part
	LDP=(ch&0b11110000);
	LCP|=1<<LCD_RS;
	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);	
	LCP&=~(1<<LCD_RS);
	_delay_ms(1);
	LDP=((ch&0b00001111)<<4);
	LCP|=1<<LCD_RS;
	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);	
	LCP&=~(1<<LCD_RS);
	_delay_ms(1);

}
// ----------------------------------------------------------------------------------
void LCDsendCommand(unsigned char cmd){	//Sends Command to LCD

	//4 bit part
	LDP=(cmd&0b11110000);
	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);
	LDP=((cmd&0b00001111)<<4);	
	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);

}
// ----------------------------------------------------------------------------------
void LCDinit(void){//Initializes LCD{

	//4 bit part
	_delay_ms(15);
	LDP=0x00;
	LCP=0x00;
	LDDR|=1<<LCD_D7|1<<LCD_D6|1<<LCD_D5|1<<LCD_D4;
	LCDR|=1<<LCD_E|1<<LCD_RW|1<<LCD_RS;
   	//---------one------
	LDP=0<<LCD_D7|0<<LCD_D6|1<<LCD_D5|1<<LCD_D4; //4 bit mode
	LCP|=1<<LCD_E|0<<LCD_RW|0<<LCD_RS;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);
	//-----------two-----------
	LDP=0<<LCD_D7|0<<LCD_D6|1<<LCD_D5|1<<LCD_D4; //4 bit mode
	LCP|=1<<LCD_E|0<<LCD_RW|0<<LCD_RS;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);
	//-------three-------------
	LDP=0<<LCD_D7|0<<LCD_D6|1<<LCD_D5|0<<LCD_D4; //4 bit mode
	LCP|=1<<LCD_E|0<<LCD_RW|0<<LCD_RS;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);
	//--------4 bit--dual line---------------
	LCDsendCommand(0b00101000);
  	//-----increment address, invisible cursor shift------
	LCDsendCommand(0b00001100);

}
// ----------------------------------------------------------------------------------
//void LCDstring(char* data, unsigned char nBytes){	//Outputs string to LCD
//	register unsigned char i;
//
//	// check to make sure we have a good pointer
//	if (!data) return;
//	// print data
//	for(i=0; i<nBytes; i++)
//		LCDsendChar(data[i]);
//}
// ----------------------------------------------------------------------------------
void LCDGotoXY(unsigned char x, unsigned char y){			//Cursor to X Y position

	register unsigned char DDRAMAddr;
	// remap lines into proper order
	switch(y)	{
		case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
		case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
		case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
		case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
		default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
	}
	// set data address
	LCDsendCommand(1<<LCD_DDRAM | DDRAMAddr);
	
}
// ----------------------------------------------------------------------------------
void LCDputsXY(unsigned char x, unsigned char y, char* string){	//Outputs string to LCD

	unsigned int i=0;
	LCDGotoXY(x, y);
	do{
		LCDsendChar(string[i]);
	}while(string[++i] != '\0');

}
// ----------------------------------------------------------------------------------
void LCDputchXY(unsigned char x, unsigned char y, char ch){

	LCDGotoXY(x, y);
	LCDsendChar(ch);

}
// ----------------------------------------------------------------------------------
// ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
void LCD2decimalXY(unsigned char x, unsigned char y, unsigned int hex){ // into 4 digit decimal text

	unsigned int h;
	
	h = (hex / 1000);
	if(h != 0)	LCDputchXY(x, y, (h + 0x30));
 	else	   	LCDputchXY(x, y, (0x30));
	h = ((hex % 1000) / 100);
	if(h != 0)	LCDputchXY(x+1, y, (h + 0x30));
	else		LCDputchXY(x+1, y, (0x30));
	h = (((hex % 1000) % 100) /10);
	if(h != 0)	LCDputchXY(x+2, y, (h + 0x30));
	else		LCDputchXY(x+2, y, (0x30));
	h= (((hex % 1000) % 100) % 10);
	if(h != 0)	LCDputchXY(x+3, y, (h + 0x30));
	else		LCDputchXY(x+3, y, (0x30));
}
// ----------------------------------------------------------------------------------
//void LCD2HexXY(unsigned char x, unsigned char y, char ch){
//
//	unsigned char ddata=0x00, dch;
//
//	dch = (ch>>4)&0x0f;
//	if( dch <= 9 ) 	ddata = '0'+dch;
//	else 		ddata = 'A'+(dch-10);
//	LCDputchXY(x, y, ddata);
//
//	dch = ch&0x0f;
//	if( dch <= 9 ) 	ddata = '0'+dch;
//	else 		ddata = 'A'+(dch-10);
//	LCDputchXY(x+1, y, ddata);
//}
// ----------------------------------------------------------------------------------
//void LCD2FloatXY(unsigned char x, unsigned char y, float data){ 	// display float as 'x.y' on LCD
//
//	unsigned int i, result;
//
//	result = (int)(data*10. + 0.5);
//	i = result/10;
//	LCDputchXY(x, y, (i+'0'));
//	LCDputchXY(x+1, y,'.');
//  	i = result%10;
//	LCDputchXY(x+2, y, (i+'0'));
//}
// ----------------------------------------------------------------------------------
//void LCD2FloatXYn_m(unsigned char x, unsigned char y, float data, unsigned char n, unsigned char m){ 	
	// display float data as 'xx.x', where n=before zero point, m=below zero point
	// (m + x = 5) as it is 16 bit value and limit is 65536 or +/- 32768
	// when using dtostrf : n=total width, m=below zero

//   char LCDtext[] = "        ";
//
//	dtostrf(data, 8, 2, LCDtext);
//	LCDputsXY(x, y, LCDtext);
//}
// ----------------------------------------------------------------------------------
void Str2Float(float data, unsigned char width, unsigned char bzero, char *s){ 	
	// display float data as 'xx.x' limited width ==> XXX.XX
	// (m + x = 5) as it is 16 bit value and limit is 65536 or +/- 32768
	unsigned int i, j, n;
	float xx;

   n = 0;
	if(data < 0)		str_a[n++] = '-';
	// Now the float is converted into str in 5 digits
	xx = data;
	for(i=0; i<bzero; i++)	xx = (xx*10.);
	j = (int)(xx + 0.5);	// limit is 65536 or +/- 32768

	i = j/10000;						// X....
	if(i != 0)	str_a[n++] = (i + 0x30);
	i = (j%10000)/1000;				// .X...
	if(i != 0)	str_a[n++] = (i + 0x30);
	else{
		if(n>0 && str_a[n-1] >='1' && str_a[n-1] <='9')	
			str_a[n++] = (i + 0x30);
	}
	i = ((j%10000)%1000)/100;		// ..X..
	if(i != 0)	str_a[n++] = (i + 0x30);
	else{
		if(n>0 && str_a[n-1] >='1' && str_a[n-1] <='9')	
			str_a[n++] = (i + 0x30);
	}
	i = (((j%10000)%1000)%100)/10;	// ...X. (first digit below zero)
	if(i != 0)	str_a[n++] = (i + 0x30);
	else			str_a[n++] = (0x30);	// Last two digit - fill with zero
	i = (((j%10000)%1000)%100)%10;	// ....X (second digit below zero)
	if(i != 0)	str_a[n++] = (i + 0x30);
	else			str_a[n++] = (0x30);

	if(n > width)	n = width;
	str_a[n] = '\0';  					// Truncate string with width
	for(i=0, j=0; i<n; i++){
		if(bzero != 0 && i == (n-bzero))	
					temp[j++] = '.';
 		temp[j++] = str_a[i];
	}	
	temp[j] = '\0';
	strcpy(s, temp);
}
// -------------------------------------------------------------------------------
/*
22.10.4.10 char dtostrf (double val, signed char_width, unsigned char __-
prec, char s)
The dtostrf() function converts the double value passed in val into an ASCII
representationthat will be stored under s. The caller is responsible for
providing sufficient storage in s.
Conversion is done in the format "[-]d.ddd". The minimum field width of the
output string (including the . and the possible sign for negative values)
 is given in width, and prec determines the number of digits after the decimal
sign. width is signed value, negative for left adjustment.
The dtostrf() function returns the pointer to the converted string s.
*/
// -------------------------------------------------------------------------------
/*
22.10.4.13 char itoa (int __val, char  __s, int __radix)
Convert an integer to a string.
The function itoa() converts the integer value from val into an ASCII representation
that will be stored under s.
The caller is responsible for providing sufficient storage in s.
Conversion is done using the radix as base, which may be a number between 2
(binary conversion) and up to 36. If radix is greater than 10, the next digit
after 9 will be the letter a.
If radix is 10 and val is negative, a minus sign will be prepended.
The itoa() function returns the pointer passed as s.
*/
// -------------------------------------------------------------------------------
//Copies string from flash memory to LCD at x y position
//const uint8_t welcomeln1[] PROGMEM="AVR LCD DEMO\0";
//CopyStringtoLCD(welcomeln1, 3, 1);	
//void CopyStringtoLCD(const char *FlashLoc, unsigned char x, unsigned char y){
//
//	unsigned char i;
//	LCDGotoXY(x,y);
//	for(i=0;(unsigned char)pgm_read_byte(&FlashLoc[i]);i++)
//		LCDsendChar((unsigned char)pgm_read_byte(&FlashLoc[i]));
//}
// ----------------------------------------------------------------------------------
void LCDshiftLeft(unsigned char n){		//Scrol n of characters Right

	for (unsigned char i=0;i<n;i++)
		LCDsendCommand(0x1E);
}
// ----------------------------------------------------------------------------------
void LCDshiftRight(unsigned char n){	//Scrol n of characters Left

	for (unsigned char i=0;i<n;i++)
		LCDsendCommand(0x18);
}
// ----------------------------------------------------------------------------------
void LCDclr(void){		LCDsendCommand(1<<LCD_CLR);}	//Clears LCD{
// ----------------------------------------------------------------------------------
void LCDhome(void){		LCDsendCommand(1<<LCD_HOME);}//LCD cursor home
// ----------------------------------------------------------------------------------
void LCDcursorOn(void){ 	LCDsendCommand(0x0E);}	//displays LCD cursor
// ----------------------------------------------------------------------------------
void LCDcursorOnBlink(void){	LCDsendCommand(0x0F);}	//displays LCD blinking cursor
// ----------------------------------------------------------------------------------
void LCDcursorOFF(void){	LCDsendCommand(0x0C);}	//turns OFF cursor
// ----------------------------------------------------------------------------------
void LCDblank(void){		LCDsendCommand(0x08);}	//blanks LCD
// ----------------------------------------------------------------------------------
void LCDvisible(void){		LCDsendCommand(0x0C);}	//Shows LCD
// ----------------------------------------------------------------------------------
void LCDcursorLeft(unsigned char n){	//Moves cursor by n poisitions left

	for (unsigned char i=0;i<n;i++)
		LCDsendCommand(0x10);
}
// ----------------------------------------------------------------------------------
void LCDcursorRight(unsigned char n){	//Moves cursor by n poisitions left

	for (unsigned char i=0;i<n;i++)
		LCDsendCommand(0x14);
}
//------------------------------------------------------------------------------









